/*!
 * immutable-arrays
 * Immutable versions of normally mutable array methods
 *
 * @version v4.1.0
 * @author George Raptis <georapbox@gmail.com>
 * @homepage https://github.com/georapbox/immutable-arrays#readme
 * @repository https://github.com/georapbox/immutable-arrays.git
 * @license MIT
 */
function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

/**
 * Deletes an element from an array by its index in the array.
 *
 * @param {Array} array The original array.
 * @param {Number} index The index of the element to delete in the original array. If index is a negative number, a copy of the original array is returned.
 * @return {Array} A new array with the element removed.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = del(originalArray, 2);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'd', 'e']
 *
 * const resultArray2 = del(originalArray, -1);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray2 ['a', 'b', 'c', 'd', 'e']
 */
function del(array, index) {
  return index >= 0 ? [].concat(_toConsumableArray(array.slice(0, index)), _toConsumableArray(array.slice(index + 1))) : _toConsumableArray(array);
}

/**
 * Removes the last element from an array by returning
 * a new array instead of mutating the original one.
 *
 * @param {Array} array The original array.
 * @return {Array} A new array with the last element removed.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = pop(originalArray);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'c', 'd']
 */
function pop(array) {
  return array.slice(0, -1);
}

/**
 * Adds one or more elements to the end of an array by returning
 * a new array instead of mutating the original one.
 *
 * @param {Array} array The original array.
 * @param {*} elementN The elements to add to the end of the array.
 * @return {Array} A new array with the new entries added to the end.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = push(originalArray, 'f', 'g');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'c', 'd', 'e', 'f', 'g']
 */
function push(array) {
  for (var _len = arguments.length, elementN = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    elementN[_key - 1] = arguments[_key];
  }

  return [].concat(_toConsumableArray(array), elementN);
}

/**
 * Reverses an array (not in place).
 * The first array element becomes the last, and the last array element becomes the first.
 *
 * @param {Array} array The original array.
 * @return {Array} A new array reversed.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = reverse(originalArray);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['e', 'd', 'c', 'b', 'a']
 */
function reverse(array) {
  return _toConsumableArray(array).reverse();
}

/**
 * Removes the first element from an array.
 *
 * @param {Array} array The original array.
 * @return {Array} A new array with the first element removed.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = shift(originalArray);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['b', 'c', 'd', 'e']
 */
function shift(array) {
  return array.slice(1);
}

/**
 * Sorts the elements of an array (not in place) and returns a sorted array.
 *
 * @param {Array} array The original array.
 * @param {Function} [compareFunction] Specifies a function that defines the sort order.
 *        If omitted, the array is sorted according to each character's Unicode code point value, according to the string conversion of each element.
 * @return {Array} A new sorted array.
 * @example
 *
 * const numberArray = [20, 3, 4, 10, -3, 1, 0, 5];
 * const stringArray = ['Blue', 'Humpback', 'Beluga'];
 *
 * const resultArray = sort(numberArray, (a, b) => a - b);
 * // -> numberArray [20, 3, 4, 10, -3, 1, 0, 5]
 * // -> resultArray [-3, 0, 1, 3, 4, 5, 10, 20]
 *
 * const resultArray = sort(numberArray, (a, b) => b - a);
 * // -> numberArray [20, 3, 4, 10, -3, 1, 0, 5]
 * // -> resultArray [20, 10, 5, 4, 3, 1, 0, -3]
 *
 * const resultArray = sort(stringArray);
 * // -> stringArray ['Blue', 'Humpback', 'Beluga']
 * // -> resultArray ['Beluga', 'Blue', 'Humpback']
 *
 * const resultArray = sort(stringArray, (a, b) => a.toLowerCase() < b.toLowerCase());
 * // -> stringArray ['Blue', 'Humpback', 'Beluga']
 * // -> resultArray ['Humpback', 'Blue', 'Beluga']
 */
function sort(array, compareFunction) {
  return _toConsumableArray(array).sort(compareFunction);
}

/**
 * Removes existing elements and/or adds new elements to an array.
 *
 * @param {Array} array The original array.
 * @param {Number} [start = array.length]
 *        Zero based index at which to start changing the array.
 *        If greater than the length of the array, actual starting index will be set to the length of the array.
 * @param {Number} [deleteCount = array.length - start]
 *        An integer indicating the number of old array elements to remove.
 *        If `deleteCount` is 0, no elements are removed.
 *        If `deleteCount` is lower than 0, `deleteCount` will be equal to 0.
 *        If `deleteCount` is greater than the number of elements left in the array starting at
 *        `start`, then all of the elements through the end of the array will be deleted.
 *        If `deleteCount` is omitted, `deleteCount` will be equal to (`array.length - start`),
 *        i.e., all of the elements beginning with `start` index on through the end of the array will be deleted.
 * @param {*} [elementN]
 *        The elements to add to the array, beginning at the start index.
 *        If you don't specify any elements, will only remove elements from the array.
 * @return {Array} The result array.
 *
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray []
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, 1);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['b', 'c', 'd', 'e']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, 3);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['d', 'e']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, originalArray.length);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray []
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, -3);
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'c', 'd', 'e']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, 0, 'lorem', 'ipsum');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['lorem', 'ipsum', 'a', 'b', 'c', 'd', 'e']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, originalArray.length, 0, 'lorem', 'ipsum');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'c', 'd', 'e', 'lorem', 'ipsum']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, 0, 2, 'lorem', 'ipsum');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['lorem', 'ipsum', 'c', 'd', 'e']
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = splice(originalArray, originalArray.length - 2, 2, 'lorem', 'ipsum');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['a', 'b', 'c', 'lorem', 'ipsum']
 */
function splice(array) {
  var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : array.length;
  var deleteCount = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : array.length - start;

  for (var _len = arguments.length, elementN = new Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    elementN[_key - 3] = arguments[_key];
  }

  return [].concat(_toConsumableArray(array.slice(0, start)), elementN, _toConsumableArray(array.slice(start + (deleteCount < 0 ? 0 : deleteCount))));
}

/**
 * Adds one or more elements to the beginning of an array.
 *
 * @param {Array} array The original array.
 * @param {*} elementN [description] The elements to add to the front of the array.
 * @return {Array} A new array with the new elements added to the front.
 * @example
 *
 * const originalArray = ['a', 'b', 'c', 'd', 'e'];
 * const resultArray = unshift(originalArray, 'f', 'g');
 * // -> originalArray ['a', 'b', 'c', 'd', 'e']
 * // -> resultArray ['f', 'g', 'a', 'b', 'c', 'd', 'e']
 */
function unshift(array) {
  for (var _len = arguments.length, elementN = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    elementN[_key - 1] = arguments[_key];
  }

  return [].concat(elementN, _toConsumableArray(array));
}

export { del, pop, push, reverse, shift, sort, splice, unshift };
